/***************************************************************************
*   Copyright (C) 2003 by Hideki Ikemoto                                  *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include "kitahtmlpart.h"

#include "kitathreadview.h"
#include "kitahtmlview.h"
#include "kitadomtree.h"

#include <kpopupmenu.h>
#include <klocale.h>
#include <khtmlview.h>
#include <dom/html_element.h>
#include <khtml_events.h>

#include <qapplication.h>
#include <qclipboard.h>

#include "libkita/thread.h"
#include "libkita/access.h"

enum {
    ID_COPY_Link,
    ID_Home_Link,
    ID_Temp_Link,
    ID_Koko_Link,
    ID_End_Link,
    ID_SAVEC_Link,
    ID_Back_Link,
    ID_Copy_Str,
    ID_Ext_Str,
    ID_Del_IMGCache,
    ID_COPY_IMGCachepath,
    ID_DEL_ThreadLog
};


KitaHTMLPart::KitaHTMLPart( QWidget* parent, const char* name )
        : KHTMLPart( new KitaHTMLView( this, parent, name ) )
        , m_parent( parent )
	  , m_thread ( 0)
{
    connect( this, SIGNAL( popupMenu( const QString&, const QPoint& ) ),
             SLOT( slotPopupMenu( const QString&, const QPoint& ) ) );

    m_popupMenu = new KPopupMenu( view() );
    m_domtree = NULL;
    m_threadView = NULL;
    
    begin();
    write( " " );
    end();
    show();
}


KitaHTMLPart::~KitaHTMLPart()
{
    if(m_popupMenu) delete m_popupMenu;
}

bool KitaHTMLPart::gotoAnchor( const QString& name )
{
    pushCurrentPosition();
    return KHTMLPart::gotoAnchor( name );
}


/* setup */
void KitaHTMLPart::setupEx( KitaThreadView* threadView, KitaDomTree* domtree,
                            Kita::Thread* thread )
{
    m_threadView = threadView;
    m_domtree = domtree;
    m_thread = thread;
    m_anchorStack.clear();
}


void KitaHTMLPart::pushCurrentPosition()
{
    QString anchor, ancundermouse;
    DOM::Node node;

    /* find the current node */
    node = nodeUnderMouse();
    while ( node != NULL && node.nodeName().string() != "div" )
        node = node.parentNode();
    if ( node == NULL ) return ;
    anchor = static_cast<DOM::Element>( node ).getAttribute( "id" ).string();

    m_anchorStack += anchor;
}


/* jump to kokomade yonda */
void KitaHTMLPart::gotoKokoyon()
{
    if(!m_domtree) return;

    int num = m_domtree->getKokoyonNum() - 1;

    while ( num > 0 ) {
        if ( m_domtree->isResShown( num ) ) break;
        num--;
    }

    KHTMLPart::gotoAnchor( QString().setNum( num ) );
}



/*-------------------------------------------------------------------*/

/* public slots */

bool KitaHTMLPart::gobackAnchor()
{
    QString anchor;

    if ( m_anchorStack.empty() ) return FALSE;

    anchor = m_anchorStack.last();
    m_anchorStack.pop_back();

    return KHTMLPart::gotoAnchor( anchor );
}



/*--------------------------------------------------*/

/* private slot */

/* show popup menu */
void KitaHTMLPart::slotPopupMenu( const QString& url, const QPoint& point )
{
    if(!m_threadView) return;
    if(!m_domtree) return;
    
    QString str;
    DOM::Node node = nodeUnderMouse();

    m_popupMenu->clear();
    
    if ( !m_anchorStack.empty() ) {
        str = "Back (" + m_anchorStack.last() + ")";
        m_popupMenu->insertItem( str, ID_Back_Link );
        m_popupMenu->insertSeparator();
    }
    
    m_popupMenu->insertItem( "Home", ID_Home_Link );
    
    if ( m_domtree->isMae100Shown() ) {
        m_popupMenu->insertItem( "template", ID_Temp_Link );
    }
    
    if ( m_domtree->getKokoyonNum() ) {
        str = "Kokomade Yonda (" + QString().setNum( m_domtree->getKokoyonNum() ) + ")";
        m_popupMenu->insertItem( str, ID_Koko_Link );
    }
    
    m_popupMenu->insertItem( "End", ID_End_Link );
    
    if ( hasSelection() ) {
        m_popupMenu->insertSeparator();
        m_popupMenu->insertItem( "Copy", ID_Copy_Str );
    }
    
    if ( url != NULL ) {
        m_popupMenu->insertSeparator();
        m_popupMenu->insertItem( i18n( "Copy Link Location" ), ID_COPY_Link );

    }
    
    if ( m_threadView->getViewMode() == VIEWMODE_PARENT ) {
        m_popupMenu->insertSeparator();
        m_popupMenu->insertItem( i18n( "Clear Log and Reload" ), ID_DEL_ThreadLog );
    }

    QClipboard* clipboard = QApplication::clipboard();

    switch ( m_popupMenu->exec( point ) ) {
    case ID_COPY_Link:
        clipboard->setText( url , QClipboard::Clipboard );
        clipboard->setText( url , QClipboard::Selection );
        break;
    case ID_Home_Link: gotoAnchor( "header" ); break;
    case ID_Temp_Link: gotoAnchor( "mae100" ); break;
    case ID_Koko_Link: gotoKokoyon();break;
    case ID_End_Link: gotoAnchor( "footer" ); break;
    case ID_Back_Link: gobackAnchor(); break;

    case ID_Copy_Str:
        clipboard->setText( selectedText(), QClipboard::Clipboard );
        break;

    case ID_DEL_ThreadLog:
        if ( Kita::Access::deleteLog( m_thread, m_parent ) )
            m_threadView->showThread( m_thread );
        break;

    default:
        break;
    }
}


/*----------------------------------------------------*/

/* protected functions */

void KitaHTMLPart::khtmlMousePressEvent( khtml::MousePressEvent* e )
{
    m_pushctrl = FALSE;
    m_pushmidbt = FALSE;

    /* check if ctrl key is pushed */
    if ( e->url() != NULL ) {

        if ( e->qmouseEvent() ->state() & Qt::ControlButton ) {
            m_pushctrl = TRUE;
        }
        if ( e->qmouseEvent() ->button() & Qt::MidButton ) {
            m_pushmidbt = TRUE;
        }

        if ( m_pushctrl || m_pushmidbt ) {
            KURL kurl;
            if ( e->url().string().at( 0 ) == '#' ) {
                kurl = KURL( m_thread->datURL(), e->url().string() );
            } else {
                kurl = KURL( m_thread->boardURL(), e->url().string() );
            }

            if ( m_pushctrl ) {
                emit browserExtension() ->openURLRequest( kurl );
            }

            return ;
        }
    }
    KHTMLPart::khtmlMousePressEvent( e );
}

void KitaHTMLPart::khtmlMouseMoveEvent( khtml::MouseMoveEvent* e )
{
    KHTMLPart::khtmlMouseMoveEvent( e );
}

void KitaHTMLPart::khtmlMouseReleaseEvent( khtml::MouseReleaseEvent *e )
{
    if ( e->qmouseEvent() ->button() & Qt::RightButton ) {

        /* popup here & return */
        slotPopupMenu( e->url().string(), e->qmouseEvent() ->globalPos() );
        return ;
    }
    KHTMLPart::khtmlMouseReleaseEvent( e );
}
